// Copyright (C) 2023 David Sugar, Tycho Softworks
// This code is licensed under MIT license

package main

import (
	"strconv"
	"syscall"

	"github.com/gofiber/fiber/v2"

	"apollo/internal/lunar"
)

func deleteLine(ctx *fiber.Ctx) error {
	ext := ctx.Params("id")
	id, err := strconv.Atoi(ext)
	if err != nil {
		id = 0
	}

	if ext != ctx.FormValue("line") {
		return ctx.Status(fiber.StatusBadRequest).SendString("line does not match id")
	}

	lunar.RemoveLine(id)
	syscall.Kill(syscall.Getpid(), syscall.SIGHUP)
	return ctx.Redirect("/lines", fiber.StatusSeeOther)
}

func passwdLine(ctx *fiber.Ctx) error {
	ext := ctx.Params("id")
	id, err := strconv.Atoi(ext)
	if err != nil {
		id = 0
	}

	line := lunar.GetLine(id)
	if line == nil {
		return ctx.Status(fiber.StatusNotFound).SendString("Line is invalid")
	}

	if !line.Editable {
		return ctx.Status(fiber.StatusBadRequest).SendString("Custom lines not changable")
	}

	save := lunar.SavedLine(id)
	passwd := ctx.FormValue("pass")
	verify := ctx.FormValue("verify")

	if passwd != verify {
		return ctx.Status(fiber.StatusBadRequest).SendString("Password does not match verify.")
	}

	if len(passwd) == 0 {
		return ctx.Status(fiber.StatusBadRequest).SendString("Password not set.")
	}

	save.MD5 = ""
	save.SHA256 = ""
	save.Secret = ""

	if lunar.HasMD5() {
		save.MD5 = lunar.ComputeMD5(ext, passwd)
	}

	if lunar.HasSHA256() {
		save.SHA256 = lunar.ComputeSHA256(ext, passwd)
	}

	lunar.UpdateLine(id, save)
	syscall.Kill(syscall.Getpid(), syscall.SIGHUP)
	return ctx.Redirect("/lines", fiber.StatusSeeOther)
}

func postNewLine(ctx *fiber.Ctx) error {
	ext := ctx.FormValue("ext")
	id, _ := strconv.Atoi(ext)

	if lunar.ExistsLine(id) {
		return ctx.Status(fiber.StatusBadRequest).SendString("Line already exists")
	}

	save := &lunar.Line{}
	save.Type = ctx.FormValue("type")
	save.Display = ctx.FormValue("display")

	count := ctx.FormValue("lines")
	lines, err := strconv.Atoi(count)
	save.Lines = uint16(lines)

	if err != nil {
		return ctx.Status(fiber.StatusBadRequest).SendString(err.Error())
	}

	passwd := ctx.FormValue("newp")
	verify := ctx.FormValue("verify")

	if passwd != verify {
		return ctx.Status(fiber.StatusBadRequest).SendString("Password does not match verify.")
	}

	if len(passwd) > 0 {
		if lunar.HasMD5() {
			save.MD5 = lunar.ComputeMD5(ext, passwd)
		}

		if lunar.HasSHA256() {
			save.SHA256 = lunar.ComputeSHA256(ext, passwd)
		}
	}

	lunar.UpdateLine(id, save)
	syscall.Kill(syscall.Getpid(), syscall.SIGHUP)
	return ctx.Redirect("/lines", fiber.StatusSeeOther)
}

func postLine(ctx *fiber.Ctx) error {
	ext := ctx.Params("id")
	id, err := strconv.Atoi(ext)
	if err != nil {
		id = 0
	}

	line := lunar.GetLine(id)
	if line == nil {
		return ctx.Status(fiber.StatusNotFound).SendString("Line is invalid")
	}

	if !line.Editable {
		return ctx.Status(fiber.StatusBadRequest).SendString("Custom lines not changable")
	}

	save := lunar.SavedLine(id)

	if ctx.FormValue("type") != line.Type {
		save.Type = ctx.FormValue("type")
	}

	if ctx.FormValue("display") != line.Display {
		save.Display = ctx.FormValue("display")
	}

	if ctx.FormValue("name") != line.Name {
		save.Name = ctx.FormValue("name")
	}

	if ctx.FormValue("email") != line.EMail {
		save.EMail = ctx.FormValue("email")
	}

	if ctx.FormValue("cabling") != line.Cabling {
		save.Cabling = ctx.FormValue("cabling")
	}

	if ctx.FormValue("location") != line.Location {
		save.Location = ctx.FormValue("location")
	}

	count := ctx.FormValue("lines")
	lines, err := strconv.Atoi(count)
	if uint16(lines) != line.Lines {
		save.Lines = uint16(lines)
	}

	if err != nil {
		return ctx.Status(fiber.StatusBadRequest).SendString(err.Error())
	}

	lunar.UpdateLine(id, save)
	syscall.Kill(syscall.Getpid(), syscall.SIGHUP)
	return ctx.Redirect("/lines", fiber.StatusSeeOther)
}
